#include <stdio.h>
#include "compass.h"
#include "esp_log.h"
static gpio_num_t i2c_gpio_sda = 18;
static gpio_num_t i2c_gpio_scl = 19;
static uint32_t i2c_frequency = 100000;
static i2c_port_t i2c_port = I2C_NUM_0;
static int16_t xzy[3] = { 0 };
static const char *TAG = "COMPASS";
float g_xzy[3] = { 0.0f };
static unsigned selected_gain = 0;
FP_AXIS_READ on_axis_read;
void set_reg(uint8_t addr, uint8_t value) {
	i2c_cmd_handle_t cmd = i2c_cmd_link_create();
	i2c_master_start(cmd);
	i2c_master_write_byte(cmd, CHIP_ADDR_WRITE,
	ACK_CHECK_EN);
	i2c_master_write_byte(cmd, addr, ACK_CHECK_EN);
	i2c_master_write_byte(cmd, value, ACK_CHECK_EN);
	i2c_master_stop(cmd);
	esp_err_t ret = i2c_master_cmd_begin(i2c_port, cmd, 500 / portTICK_RATE_MS);
	i2c_cmd_link_delete(cmd);
	if (ret == ESP_ERR_TIMEOUT)
		ESP_LOGD(TAG, "TIMEOUT");
	if (ret == ESP_OK) {
		ESP_LOGD(TAG, "REG SET %X", addr);
	}
}
void get_reg(uint8_t addr, uint8_t *buffer) {
	i2c_cmd_handle_t cmd = i2c_cmd_link_create();
	i2c_master_start(cmd);
	i2c_master_write_byte(cmd, CHIP_ADDR_WRITE, ACK_CHECK_EN);
	i2c_master_write_byte(cmd, addr, ACK_CHECK_EN);
	i2c_master_start(cmd);
	i2c_master_write_byte(cmd, CHIP_ADDR_READ, ACK_CHECK_EN);
	i2c_master_read_byte(cmd, buffer, NACK_VAL);
	i2c_master_stop(cmd);
	esp_err_t ret = i2c_master_cmd_begin(i2c_port, cmd, 500 / portTICK_RATE_MS);
	i2c_cmd_link_delete(cmd);
	if (ret == ESP_ERR_TIMEOUT)
		ESP_LOGD(TAG, "TIMEOUT");
	if (ret == ESP_OK) {
		ESP_LOGD(TAG, "REG GET %X", addr);
	}
}

uint8_t config_a(const uint8_t num_of_sample_averaged,
		const uint8_t data_output_rate) {
	uint8_t conf = 0;
	conf |= num_of_sample_averaged << 5;
	conf |= data_output_rate << 2;
//	conf|=measurement_configuration;
	return conf;
}

uint8_t config_b(uint8_t gain) {
	uint8_t conf = 0;
	conf |= gain << 5;
	return conf;
}
void init_compass(FP_AXIS_READ fn) {
	on_axis_read = fn;
	i2c_driver_install(i2c_port, I2C_MODE_MASTER, 0, 0, 0);
	i2c_config_t conf = { .mode = I2C_MODE_MASTER, .sda_io_num = i2c_gpio_sda,
			.sda_pullup_en = GPIO_PULLUP_ENABLE, .scl_io_num = i2c_gpio_scl,
			.scl_pullup_en = GPIO_PULLUP_ENABLE, .master.clk_speed =
					i2c_frequency, };
	i2c_param_config(i2c_port, &conf);
	uint8_t byte_buffer = 0;
	byte_buffer = config_a(SAMPLE_NUM_8, DATA_RATE_75);
	set_reg(CA_REG_ADDR, byte_buffer);
	get_reg(CA_REG_ADDR, &byte_buffer);
	ESP_LOGD(TAG, "%X", byte_buffer);

	byte_buffer = config_b(GAIN_BITS[selected_gain]);
	set_reg(CB_REG_ADDR, byte_buffer);
	get_reg(CB_REG_ADDR, &byte_buffer);
	ESP_LOGD(TAG,"%X", byte_buffer);

	set_reg(MODE_REG_ADDR, 0);
	get_reg(MODE_REG_ADDR, &byte_buffer);
	ESP_LOGD(TAG, "%X", byte_buffer);
}
void read_axis() {
	uint8_t buffer[6] = { 0 };
	i2c_cmd_handle_t cmd = i2c_cmd_link_create();
	i2c_master_start(cmd);
	i2c_master_write_byte(cmd, CHIP_ADDR_WRITE, ACK_CHECK_EN);
	i2c_master_write_byte(cmd, XZY_REG_ADDRS[0], ACK_CHECK_EN);
	i2c_master_start(cmd);
	i2c_master_write_byte(cmd, CHIP_ADDR_READ, ACK_CHECK_EN);
	for (int i = 0; i < 5; i++) {
		i2c_master_read_byte(cmd, buffer + i, ACK_VAL);
	}
	i2c_master_read_byte(cmd, buffer + 5, NACK_VAL);
	i2c_master_stop(cmd);
	esp_err_t ret = i2c_master_cmd_begin(i2c_port, cmd,
			1000 / portTICK_RATE_MS);
	i2c_cmd_link_delete(cmd);
	if (ret == ESP_ERR_TIMEOUT)
		ESP_LOGD(TAG, "TIMEOUT");
	int16_t b = 0;
	if (ret == ESP_OK) {
		for (int i = 0; i < 6; i++) {
			if (i % 2 == 0) {
				b = 0;
				xzy[i] = buffer[i] << 8;
				b = buffer[i] << 8;
			} else {
				xzy[i - 1] |= buffer[i];
				b |= buffer[i];
				ESP_LOGD(TAG, "iter%d: %u ## %d", i, b, b);
			}
		}
	}
}
void compass_task(void *args) {
	while (1) {
		read_axis();
		for (int i = 0; i < 3; i++)
			g_xzy[i] = ((float) xzy[i]) / GAINS[selected_gain];
		on_axis_read(xzy[0], xzy[2], xzy[1], g_xzy[0], g_xzy[2], g_xzy[1]);
		vTaskDelay(1000 / portTICK_PERIOD_MS);
	}
}
