#include <math.h>
#include "freertos/FreeRTOS.h"
#include "esp_log.h"
#include "compass.h"
#include "display.h"
#include "inputs.h"
static const char *TAG = "NAVIGATOR";
static QueueHandle_t input_to_display;
static void on_axis_read(int16_t rx, int16_t ry, int16_t rz, float x, float y,
		float z) {
	float theta = atan2(y, x);
	DisplayMsg msg = { .type = TYPE_COMPASS_READ, .rx = rx, .ry = ry, .rz = rz,
			.x = x, .y = y, .z = z, .msg_len = 0, .theta = theta };
	uint8_t count = 5;
	while (xQueueSend(input_to_display, &msg, 100) != pdTRUE) {
		if (--count == 0) {
			ESP_LOGE(TAG,
					"Error sending msg! Reach maximum num of send attempts!");
			break;
		}
	}
	ESP_LOGD(TAG, "MEASURE:");
	ESP_LOGD(TAG, "x:%d,y:%d,z:%d", rx, ry, rz);
	ESP_LOGD(TAG, "x:%f,y:%f,z:%f", x, y, z);
}

static void input_received(uint8_t action) {
	DisplayMsg msg =
			{ .type = TYPE_MENU_NAVIGATE, .page_action = MAINTAIN_PAGE };
	if (action == Y_NEG)
		msg.item_action = NEXT_ITEM;
	else if (action == Y_POS)
		msg.item_action = PREV_ITEM;
	else if (action == KEY_PRESS) {
		msg.item_action = SELECT_ITEM;
	} else
		msg.item_action = UNDEFINED;
	if (msg.item_action == IGNORE)
		return;
	if (msg.item_action != UNDEFINED)
		xQueueSend(input_to_display, &msg, 0);
	else
		ESP_LOGW(TAG, "Unknown display input:%u", action);
}
void main_task(void *args) {
	uint32_t s = esp_get_minimum_free_heap_size();
	while (1) {
		ESP_LOGI(TAG, "Minimum free heap size:%u bytes", s);
		vTaskDelay(pdMS_TO_TICKS(5000));
	}
}
void app_main(void) {
	ESP_LOGD(TAG, "float_size:%u", sizeof(float));
	init_compass(on_axis_read);
	ESP_LOGD(TAG, "Compass initialized.");
	init_display(&input_to_display);
	ESP_LOGD(TAG, "Display initialized.");
	init_inputs(input_received);
	vTaskDelay(pdMS_TO_TICKS(500));
	xTaskCreate(compass_task, "compass", 4096, NULL, 5, NULL);
	xTaskCreate(display_task, "display", 4096, NULL, 5, NULL);
	xTaskCreate(input_task, "inputs", 4096, NULL, 5, NULL);
	xTaskCreate(main_task, "main", 4096, NULL, 5, NULL);
}

